phosh-top-panel.debug {
    background-color: red;
    animation: red-pulse 2s linear infinite;
}

.phosh-fader-default-fade {
    animation-duration: 500ms;
}

@keyframes red-pulse
{
    0%   {background-color: @phosh_bg_color;}
    50% {background-color: red;}
    100%   {background-color: @phosh_bg_color;}
}

phrog-user-session-page list {
    background-color: transparent;
}

phrog-user-session-page .users row:not(:selected) {
    background-color: @phosh_action_bg_color;
}

phrog-user-session-page .users row:selected {
    background-color: @theme_selected_bg_color;
}

phrog-user-session-page .sessions {
    background-color: @phosh_action_bg_color;
}

phrog-user-session-page row:first-child {
    -gtk-outline-top-left-radius: 7px;
    -gtk-outline-top-right-radius: 7px;
    border-top-left-radius: 8px;
    border-top-right-radius: 8px;
}

phrog-user-session-page row:not(:last-child) {
    border-width: 1px 1px 0px 1px;
}

phrog-user-session-page row:last-child {
    -gtk-outline-bottom-left-radius: 7px;
    -gtk-outline-bottom-right-radius: 7px;
    border-bottom-left-radius: 8px;
    border-bottom-right-radius: 8px;
}
